! The icMRCCSD(T) potential energy surface for the FHCl system
      program test_fhcl_pes
      implicit none

      DOUBLE PRECISION :: DIST(3)  ! ABC convention: (R_HF, R_HCl, R_ClF)
      DOUBLE PRECISION :: POT

      double precision, external :: V

      DIST(1) = 100.0d0
      DIST(2) = 2.410d0
      DIST(3) = 100.0d0
      CALL FHCL_MRCC(DIST,POT)
      write(6,*) 'The energy at the entance channel is:'
      write(6,*) POT
      write(6,*)

      DIST(1) = 1.733d0
      DIST(2) = 100.0d0
      DIST(3) = 100.0d0
      CALL FHCL_MRCC(DIST,POT)
      write(6,*) 'The reaction exothermicity is:'
      write(6,*) POT
      write(6,*)

      DIST(1) = 2.919600d0
      DIST(2) = 2.485202d0
      DIST(3) = 4.466868d0
      CALL FHCL_MRCC(DIST,POT)
      write(6,*) 'The barrier height is:'
      write(6,*) POT




      end program test_fhcl_pes


C WRAPER FOR THE ACTUAL FUNCTION.
C IN THE FIRST CALL, READ THE PARAMETERS
      SUBROUTINE FHCL_MRCC( DIST, POT)
      IMPLICIT NONE
      
      DOUBLE PRECISION :: DIST(3)        ! ABC convention: (R_HF, R_HCl, R_ClF)
      DOUBLE PRECISION :: POT
      DOUBLE PRECISION :: R(3)           ! PES convention: (R_HF, R_ClF, R_HCl)
      DOUBLE PRECISION, EXTERNAL :: V

      DOUBLE PRECISION, PARAMETER :: TOeV = 27.211383860484776D0
      DOUBLE PRECISION, PARAMETER :: TOkcalmol = 627.5091977345D0

      DOUBLE PRECISION, PARAMETER :: SHIFT = 0.171496792D0 ! De for HCl at icMRCCSD(T)-ccw/CBS

      R(1) = DIST(1) ! From the ABC program to the PES convention
      R(2) = DIST(3)
      R(3) = DIST(2)

      POT = V(R)
      POT = POT + SHIFT
      POT = POT*TOkcalmol !*TOeV

      RETURN

      END SUBROUTINE FHCL_MRCC

! INIT
!----------
      SUBROUTINE VINIT
      
      double precision ::
     &     coef_2b_1(0:20),
     &     coef_2b_2(0:20),
     &     coef_2b_3(0:20)
      integer :: N_1, N_2, N_3
      double precision :: coef_3b(0:2000)
      integer :: M,    n_coef,    L
      integer :: M_TS, n_coef_TS, L_TS
      double precision :: R1_0, R2_0, R3_0, R1_0_TS, R2_0_TS, R3_0_TS
      integer, external :: GET_N_COEF

      character(len=150), parameter :: 
     &     file_2b_1  = 'coef_FH',
     &     file_2b_2  = 'coef_ClF',
     &     file_2b_3  = 'coef_HCl'

      character(len=150), parameter :: 
     &     file_3b    = 'coef_3b'

      common/vparams/
     &     coef_2b_1, coef_2b_2, coef_2b_3,
     &     coef_3b,
     &     R1_0,    R2_0,    R3_0,
     &     R1_0_TS, R2_0_TS, R3_0_TS,
     &     N_1, N_2, N_3,
     &     M,    n_coef,    L,
     &     M_TS, n_coef_TS, L_TS

      N_1 = 9
      N_2 = 9
      N_3 = 9

      M = 10
      L = 0
      n_coef = GET_N_COEF( M)
      M_TS = 7
      L_TS = 2
      n_coef_TS = GET_N_COEF( M_TS)

      R1_0 = 1.741284d0
      R2_0 = 3.104319d0
      R3_0 = 2.414313d0

      R1_0_TS = 1.741284d0
      R2_0_TS = 3.104319d0
      R3_0_TS = 2.414313d0

      CALL GET_TWO_BODY( file_2b_1, coef_2b_1, N_1)
      CALL GET_TWO_BODY( file_2b_2, coef_2b_2, N_2)
      CALL GET_TWO_BODY( file_2b_3, coef_2b_3, N_3)

      CALL GET_THREE_BODY(file_3b, coef_3b, n_coef, L,
     &     n_coef_TS, L_TS)

      END SUBROUTINE VINIT

!-------------------------------------------
! The potential
!--------
      double precision function V(R)
      implicit none

      double precision :: R(3)

      double precision ::
     &     coef_2b_1(0:20),
     &     coef_2b_2(0:20),
     &     coef_2b_3(0:20)
      integer :: N_1, N_2, N_3
      double precision :: coef_3b(0:2000)
      integer :: M,    n_coef,    L
      integer :: M_TS, n_coef_TS, L_TS
      double precision :: two_b_1, two_b_2, two_b_3, three_b
      double precision, external :: V2, V3
      double precision :: R1_0, R2_0, R3_0, R1_0_TS, R2_0_TS, R3_0_TS
      double precision, parameter :: alpha = 1.02403d0

      common/vparams/
     &     coef_2b_1, coef_2b_2, coef_2b_3,
     &     coef_3b,
     &     R1_0,    R2_0,    R3_0,
     &     R1_0_TS, R2_0_TS, R3_0_TS,
     &     N_1, N_2, N_3,
     &     M,    n_coef,    L,
     &     M_TS, n_coef_TS, L_TS

      integer, save :: ncall
      data ncall/-1/
      
      if (ncall.EQ.-1) then
       call VINIT
       ncall = 0
      end if

      if (R(1).LT.1.0d0 .OR. R(2).LT.2.25d0 .OR. R(3).LT.1.5d0) then
       V = 100.0d0
       RETURN
      end if

      two_b_1 = V2(R(1), coef_2b_1, N_1)
      two_b_2 = V2(R(2), coef_2b_2, N_2)
      two_b_3 = V2(R(3), coef_2b_3, N_3)
      three_b = V3(R(1), R(2), R(3), coef_3b,
     &     M, n_coef, L,
     &     M_TS, n_coef_TS, L_TS,
     &     R1_0, R2_0, R3_0,
     &     R1_0_TS, R2_0_TS, R3_0_TS)

      V = two_b_1 + two_b_2 + two_b_3 + three_b*alpha

      END FUNCTION



!-------------------------------------------
! TWO BODY POTENTIAL
!-----------
! as in deskevichJCP06_124_224303
!-----------
!
!     coef(0) = beta_1
!     coef(1) = beta_2
!     coef(2) = C_0
!     ...
!     coef(i) = C_{i-2}
!     ...
!     coef(N+2) = C_N

!     Get coefficients from file
!----------
      subroutine GET_TWO_BODY( file_name, coef, N)
      implicit none
      character(len=150) :: file_name
      double precision :: coef(0:20)
      integer :: N
      integer :: i
      open(unit=100, status='OLD', action='READ',
     &     file=file_name)
      if (N > 18) then
       print*, "Error in GET_TWO_BODY: N is too large."
       stop
      end if
      do i = 0, N+2
       read(100,*) coef(i)
      end do
      close(100)
      end subroutine GET_TWO_BODY

!     Two-body function
!----------
      double precision function V2(R, coef, N)
      implicit none
      double precision :: R
      double precision :: coef(0:20)
      integer :: N
      integer :: i
      double precision :: rho, rho_i
      rho = R*exp( -coef(1)*R)
      rho_i = rho
      V2 = coef(2)*exp(-coef(0)*R)/R
      do i = 1, N
       V2 = V2 + coef(i+2)*rho_i
       rho_i = rho_i*rho
      end do
      end function V2

!-------------------------------------------
! THREE BODY POTENTIAL
!-----------
! as in petersonJCP99_111_7448
!-----------
!
!     coef(0) = beta_1
!     coef(1) = beta_2
!     coef(2) = beta_3
!     coef(3) = C_1    (D_0110)
!     coef(4) = C_2
!     ...
!     coef(i) = C_{i-2}
!     ...
!     coef(n_coef+2) = C_n_coef
!     ...
!     ...
!     coef( ) = C_1   (D_011L)
!     coef( ) = C_2
!     ...
!     coef( ) = C_{i-2}
!     ...
!     coef(L*n_coef+2) = C_n_coef
!

!     Get coefficients from file
!----------
      subroutine GET_THREE_BODY( file_name, coef, n_coef, L,
     &     n_coef_TS, L_TS)
      implicit none
      character(len=150) :: file_name
      double precision :: coef(0:2000)
      integer :: n_coef, L, n_coef_TS, L_TS
      integer :: i
      open(unit=100, status='OLD', action='READ',
     &     file=file_name)
      if (n_coef*(L+1) + n_coef_TS*(L_TS+1) + 5 > 2000) then
       print*, "Error in GET_THREE_BODY: n_coef*(L+1) + "//
     &      "n_coef_TS*(L_TS+1) is too large."
       stop
      end if
      do i = 0, 5 + n_coef*(L+1) + n_coef_TS*(L_TS+1)
       read(100,*) coef(i)
      end do
      close(100)
      end subroutine GET_THREE_BODY

!     Calculate the number of coeffcients (for each l)
!----------
      integer function GET_N_COEF( M)
      implicit none
      integer :: M
      integer :: i, j, k
      GET_N_COEF = 0
      do i = 0, M
       do j = 0, M
        do k = 0, M
         if (i + j + k .LE. M) then
          if (i*j .GT. 0 .OR. j*k .GT. 0 .OR. k*i .GT. 0) then
           GET_N_COEF = GET_N_COEF + 1
          end if
         end if
        end do
       end do
      end do
      end function GET_N_COEF


!     Three-body function
!----------
      double precision function V3(R1, R2, R3, coef, 
     &     M, n_coef, L,
     &     M_TS, n_coef_TS, L_TS,
     &     R1_0, R2_0, R3_0,
     &     R1_0_TS, R2_0_TS, R3_0_TS)
      implicit none
      double precision :: R1, R2, R3
      double precision :: coef(0:1000)
      integer :: M, n_coef, L
      integer :: M_TS, n_coef_TS, L_TS
      double precision :: R1_0, R2_0, R3_0
      double precision :: R1_0_TS, R2_0_TS, R3_0_TS

      double precision, parameter :: TS_1 = 0.493715d0
      double precision, parameter :: TS_2 = 2.529145d0
      double precision, parameter :: TS_3 = 1.985675d0
      double precision :: r_per_1, r_per_2, r_per_3, dist, x
      double precision :: rho1, rho1_i, rho2, rho2_j, rho3, rho3_k
      double precision :: sum_cos, sum_cos_l, V3_tmp
      integer :: im, i, j, k, il

!                 rFH  rClF rHCl
      r_per_1 = ( R1 - R2 + R3)/2 ! rH
      r_per_2 = ( R1 + R2 - R3)/2 ! rF
      r_per_3 = (-R1 + R2 + R3)/2 ! rCl
      
      sum_cos = (R1**2)*r_per_3 +
     &          (R2**2)*r_per_1 +
     &          (R3**2)*r_per_2
      sum_cos = 1.0d0 + sum_cos/(R1*R2*R3)

      dist = sqrt(
     &     (r_per_1-TS_1)**2 + 
     &     (r_per_2-TS_2)**2 +
     &     (r_per_3-TS_3)**2)
      x = exp(-1.0d0*dist**2)

!     First part: long range (LR)
      rho1 = exp( -coef(0)*(R1 - R1_0))
      rho2 = exp( -coef(1)*(R2 - R2_0))
      rho3 = exp( -coef(2)*(R3 - R3_0))
      rho1 = exp( -R1_0/R1)*rho1
      rho2 = exp( -R2_0/R2)*rho2
      rho3 = exp( -R3_0/R3)*rho3
      V3 = 0.0d0
      do il = 0,L
       sum_cos_l = sum_cos**il
       i = 0
       j = 0
       k = 0
       rho1_i = 1.0d0
       rho2_j = 1.0d0
       rho3_k = 1.0d0
       V3_tmp = 0.0d0
       do im = 0, n_coef-1
        k = k + 1
        rho3_k = rho3_k*rho3
        if (i + j + k .GT. M) then
         k = 0
         rho3_k = 1.0d0
         j = j + 1
         rho2_j = rho2_j*rho2
        end if
        if ((j + k .EQ. 0) .OR. (i + k .EQ. 0)) then
         k = 1
         rho3_k = rho3
        end if
        if (i + j + k .GT. M) then
         j = 0
         rho2_j = 1.0
         i = i + 1
         rho1_i = rho1_i*rho1
        end if
        if ((j + k .EQ. 0) .OR. (i + k .EQ. 0)) then
         k = 1
         rho3_k = rho3
        end if
        if ((j + i .EQ. 0)) then
         j = 1
         rho2_j = rho2
        end if
        V3_tmp = V3_tmp +
     &       coef(6 + il*n_coef + im)*rho1_i*rho2_j*rho3_k
       end do
       V3 = V3 + V3_tmp*sum_cos_l
      end do
      V3 = V3*(1.0d0 - x)

!     Second part: TS region
      rho1 = exp( -coef(0+3)*(R1 - R1_0_TS))
      rho2 = exp( -coef(1+3)*(R2 - R2_0_TS))
      rho3 = exp( -coef(2+3)*(R3 - R3_0_TS))
      rho1 = exp( -R1_0_TS/R1)*rho1
      rho2 = exp( -R2_0_TS/R2)*rho2
      rho3 = exp( -R3_0_TS/R3)*rho3
      do il = 0,L_TS
       sum_cos_l = sum_cos**il
       i = 0
       j = 0
       k = 0
       rho1_i = 1.0d0
       rho2_j = 1.0d0
       rho3_k = 1.0d0
       V3_tmp = 0.0d0
       do im = 0, n_coef_TS-1
        k = k + 1
        rho3_k = rho3_k*rho3
        if (i + j + k .GT. M_TS) then
         k = 0
         rho3_k = 1.0d0
         j = j + 1
         rho2_j = rho2_j*rho2
        end if
        if ((j + k .EQ. 0) .OR. (i + k .EQ. 0)) then
         k = 1
         rho3_k = rho3
        end if
        if (i + j + k .GT. M_TS) then
         j = 0
         rho2_j = 1.0
         i = i + 1
         rho1_i = rho1_i*rho1
        end if
        if ((j + k .EQ. 0) .OR. (i + k .EQ. 0)) then
         k = 1
         rho3_k = rho3
        end if
        if ((j + i .EQ. 0)) then
         j = 1
         rho2_j = rho2
        end if
        V3_tmp = V3_tmp +
     &       coef(6 + il*n_coef_TS + im + (L+1)*n_coef)
     &       *rho1_i*rho2_j*rho3_k
       end do
       V3 = V3 + V3_tmp*sum_cos_l*x
      end do

      end function V3
